/*-----------------------------------------------------------------------

File  : cto_proofcontrol.h

Author: Stephan Schulz

Contents
 
  Object storing all information about control of the search
  process: Ordering, heuristic, similar stuff.

Changes

<1> Fri Oct 16 14:52:53 MET DST 1998
    New

-----------------------------------------------------------------------*/

#ifndef CCO_PROOFCONTROL

#define CCO_PROOFCONTROL

#include <ccl_proofstate.h>
#include <che_heuristics.h>
#include <che_lpogen.h>
#include <che_to_weightgen.h>
#include <che_to_precgen.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef struct proofcontrolcell
{
   TermOrdering      ordertype;
   TOWeightGenMethod to_weight_gen;
   TOPrecGenMethod   to_prec_gen;
   long              to_const_weight;
   OCB_p             ordering;
   HCB_p             heuristic;
   bool              use_indexing;
   RewriteLevel      forward_demod;
   bool              prefer_initial_clauses;
   bool              paramod_with_units_only;
}ProofControlCell, *ProofControl_p;



/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define ProofControlCellAlloc() \
   (ProofControlCell*)SizeMalloc(sizeof(ProofControlCell))
#define ProofControlCellFree(junk) \
   SizeFree(junk, sizeof(ProofControlCell))

ProofControl_p ProofControlAlloc();
void           ProofControlFree(ProofControl_p junk);

OCB_p ProofControlCreateOrdering(ProofState_p state, ProofControl_p
				 control); 

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/






